// Written by Robert C. Kirby
// Copyright 2005 by The University of Chicago
// Distributed under the LGPL license
// This work is partially supported by the US Department of Energy
// under award number DE-FG02-04ER25650

#include "FIAT_la.hpp"
#include "FIAT_exception.hpp"

using namespace FIAT;
using namespace blitz;

// LAPACK interface
extern "C" {
  void dgetrf_( int *m , int *n , double *a, int *lda ,
                int *ipiv , int *info );

  void dgetri_( int *n , double *a , int *lda ,
                int *ipiv , double *work , int *lwork ,
                int *info );
  void dgemm_( char *ta , char *tb ,
	       int *m , int *n, int *k ,
	       double *alpha ,
	       double *a , int *lda ,
	       double *b , int *ldb ,
	       double *beta ,
	       double *c , int *ldc );
};

namespace FIAT {

void invert( Array<double,2> &a )
{
  if (a.rows() != a.columns()) FIAT_ERROR("nonsquare matrix in invert()");
  if (!a.isStorageContiguous()) 
    FIAT_ERROR("non-contiguous storage in invert()");

  int n = a.rows();
  int info;
    
  Array<int,1> ipiv(n);

  dgetrf_( &n , &n , a.data() , &n , ipiv.data() , &info );

  int lwork = 30 * n;
  Array<double,1> work(lwork);

  dgetri_( &n , a.data() , &n , ipiv.data() , work.data() , &lwork , &info );

  return;
}

// C <-- A*B.  C is *must* be preallocated appropriately
// Blitz is row-major, so we swap transpose flag.
void mat_mat_mul( Array<double,2> &A ,
		  Array<double,2> &B ,
		  Array<double,2> &C )
{
  char t = 't';
  int lda = A.stride(0);
  int ldb = B.stride(0);
  int ldc = C.stride(0);
  double d_one = 1.0;
  double d_zero = 0.0;

  int m = A.extent(1);
  int k = A.extent(0);
  int n = B.extent(0); 

  dgemm_( &t , &t , 
	  &m, &n, &k ,
	  &d_one ,
	  A.data() , &lda ,
	  B.data() , &ldb ,
	  &d_zero ,
	  C.data() , &ldc );
  
  return;
}
}

// C <-- A*B'.  C is *must* be preallocated appropriately
// Blitz is row-major, so we swap transpose flag.
// void mat_mat_mul( Array<double,2> &A ,
// 		  Array<double,2> &B ,
// 		  Array<double,2> &C )
// {
//   char t = 't';
//   char n = 't';
//   int lda = A.stride(0);
//   int ldb = B.stride(0);
//   int ldc = C.stride(0);
//   double d_one = 1.0;
//   double d_zero = 0.0;

//   int m = A.extent(1);
//   int k = A.extent(0);
//   int n = B.extent(1);


//   dgemm_( &t , &t , 
// 	  &m, &n, &k ,
// 	  &d_one ,
// 	  A.data() , &lda ,
// 	  B.data() , &ldb ,
// 	  &d_zero ,
// 	  C.data() , &ldc );
  
//   return;
// }
		  
