// Written by Robert C. Kirby
// Copyright 2005 by The University of Chicago
// Distributed under the LGPL license
// This work is partially supported by the US Department of Energy
// under award number DE-FG02-04ER25650

#include "FIAT_lagrange.hpp"

using namespace FIAT;
using namespace blitz;

LagrangeDualBasis::LagrangeDualBasis( RCP<PolynomialBase> PB ) : 
  DualBasis()
{
  int n = PB->degree();
  int shp = PB->domain_shape();
  int td = Shapes::topological_dimension( shp );

  vector<RCP<Functional> > ls;

  RCP<Functional> delta;

  int id_cur = 0;
  for (int d=0;d<=td;d++) {
    vector<vector<int> > dim_cur;
    for (int e=0;e<Shapes::num_entities(shp,d);e++) {
      Array<double,2> xs = Shapes::make_points(shp,d,e,n);
      int np = xs.rows();
      vector<int> entity_cur;
      for (int i=0;i<np;i++) {
 	entity_cur.push_back( id_cur++ );
	Array<double,1> xcur = xs(i,Range::all());
	//delta = new PointEvaluation( PB , xcur );
	ls.push_back( new PointEvaluation( PB , xcur ) );
      }
      dim_cur.push_back( entity_cur );
    }
    eids.push_back( dim_cur );
  }

  FS = new FunctionalSet( PB , ls );

}

LagrangeElement::LagrangeElement( int shp , int n ) :
  FiniteElement()
{
  RCP<PolynomialBase> PB = new PolynomialBase( shp , n );
  RCP<ScalarPolynomialSet> PS1 = new OrthonormalScalarPolynomialSet( PB );
  DB = new LagrangeDualBasis( PB );
  Array<double,2> vinv = DB->getFunctionalSet()->outer_product( PS1 );
  invert( vinv );
  PS = new ScalarPolynomialSet( PB , vinv.transpose(1,0) );
}
