// Written by Robert C. Kirby
// Copyright 2005 by The University of Chicago
// Distributed under the LGPL license
// This work is partially supported by the US Department of Energy
// under award number DE-FG02-04ER25650

#include "FIAT_p1nc.hpp"

using namespace FIAT;
using namespace blitz;

P1NCDualBasis::P1NCDualBasis( RCP<PolynomialBase> PB ) : 
  DualBasis()
{
  int n = PB->degree();
  int shp = PB->domain_shape();
  int td = Shapes::topological_dimension( shp );

  vector<RCP<Functional> > ls;

  RCP<Functional> delta;


  eids.resize( td + 1 );
  for (int d=0;d<=td;d++) {
    eids[d].resize( Shapes::num_entities(shp,d) );
    for (int e=0;e<Shapes::num_entities(shp,d);e++ ) {
      eids[d][e].resize( 0 );
    }
  }


  // there are td+1 facets of dimension td-1
  // 3 edges on triangles and 4 faces on tets
  eids[td-1].resize(Shapes::num_entities(shp,td-1));
  for (int e=0;e<Shapes::num_entities(shp,td-1);e++) {
    eids[td-1][e].resize( 1 );
    Array<double,2> xs = Shapes::make_points(shp,td-1,e,td);
    if (xs.rows() != 1) { }
    Array<double,1> xcur = xs(0,Range::all());
    ls.push_back( new PointEvaluation( PB , xcur ) );
    eids[td-1][e][0] = e;
  }

  FS = new FunctionalSet( PB , ls );

}

P1NCElement::P1NCElement( int shp ) :
  FiniteElement()
{
  RCP<PolynomialBase> PB = new PolynomialBase( shp , 1 );
  RCP<ScalarPolynomialSet> PS1 = 
    new OrthonormalScalarPolynomialSet( PB );
  DB = new P1NCDualBasis( PB );
  Array<double,2> vinv = DB->getFunctionalSet()->outer_product( PS1 );
  invert( vinv );
  PS = new ScalarPolynomialSet( PB , vinv.transpose(1,0) );
}
